"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.dereference = void 0;
const promises_1 = require("fs/promises");
const json_schema_ref_parser_1 = require("@apidevtools/json-schema-ref-parser");
const utils_1 = require("./utils");
function dereference(schema_1, _a) {
    return __awaiter(this, arguments, void 0, function* (schema, { cwd, $refOptions, overrideHttpId, }) {
        (0, utils_1.log)('green', 'dereferencer', 'Dereferencing input schema:', cwd, schema);
        const parser = new json_schema_ref_parser_1.$RefParser();
        const myResolver = new MyResolver(overrideHttpId);
        const dereferencedPaths = new WeakMap();
        const dereferencedSchema = (yield parser.dereference(cwd, schema, Object.assign(Object.assign({}, $refOptions), { dereference: Object.assign(Object.assign({}, $refOptions.dereference), { onDereference($ref, schema) {
                    dereferencedPaths.set(schema, $ref);
                } }), resolve: {
                httpOverride: {
                    order: 1,
                    canRead(file) {
                        return myResolver.canRead(file);
                    },
                    read(file) {
                        return myResolver.read(file);
                    },
                },
            } }))); // TODO: fix types
        return { dereferencedPaths, dereferencedSchema };
    });
}
exports.dereference = dereference;
class MyResolver {
    constructor(overrideHttpId) {
        if (overrideHttpId !== null) {
            const result = [];
            for (const [originalSchemaUrl, overrideSchemaUrl] of Object.entries(overrideHttpId)) {
                result.push({ originalSchemaUrl: new URL(originalSchemaUrl), overrideSchemaUrl: new URL(overrideSchemaUrl) });
            }
            this.overrideHttpId = Object.freeze(result);
        }
        else {
            this.overrideHttpId = null;
        }
    }
    canRead(file) {
        if (this.overrideHttpId !== null) {
            for (const { originalSchemaUrl } of this.overrideHttpId) {
                const originalSchemaUrlStr = originalSchemaUrl.toString();
                if (file.url.startsWith(originalSchemaUrlStr)) {
                    return true;
                }
            }
        }
        return false;
    }
    read(file) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.overrideHttpId !== null) {
                for (const { originalSchemaUrl, overrideSchemaUrl } of this.overrideHttpId) {
                    const originalSchemaUrlStr = originalSchemaUrl.toString();
                    if (file.url.startsWith(originalSchemaUrlStr)) {
                        const overrideSchemaUrlStr = overrideSchemaUrl.toString();
                        try {
                            const newUrl = new URL(file.url.replace(originalSchemaUrlStr, overrideSchemaUrlStr));
                            const fileData = yield (0, promises_1.readFile)(newUrl);
                            return fileData;
                        }
                        catch (e) {
                            console.error(`Failed to replace '${originalSchemaUrlStr}' -> '${overrideSchemaUrlStr}'`);
                            console.error(e);
                            throw e;
                        }
                    }
                }
            }
            throw new Error('Шось пішло не так');
        });
    }
}
//# sourceMappingURL=resolver.js.map